% CALIBRATION calibrate the RFbeam V-MD3 radar sensor
%
% Use the following static IPV4 settings on your computer:
%   IP-Adress:  192.168.100.1
%   Subnet:     255.255.255.0
% Make sure that your firewall does not block the UDP packets.
%
% Place a corner reflector in front of the radar sensor.

close all;
clear;

file='mimo0.bin'; % file name for recording and playback of measurement data
mode='rfbmimofile';
Q=100; % number of measurements for averaging

% start streaming of radar sensor data
sensor=startstreaming(mode,file);

% reset calibration
sensor.caldata=ones(12,1);

reference=zeros(12,1);
for q=1:Q
    % read raw adc values from sensor
    radc=getframe(sensor);
    L=size(radc,1); % samples in fast time domain
    P=size(radc,2); % samples in slow time domain
    N=size(radc,3); % antenna elements
    % range-Doppler-plot, averaged over antenna elements
    rdfft=flip( ... % positive velocities result in negative Dopplers
        fftshift( ... % move zero Doppler to the center
            fft( .... % Doppler fft, twofold oversampling
                fft(radc,8*L,1), ... % range fft, eightfold oversampling
                2*P,2 ...
            ), ...
            2 ...
        ), ...
        2 ...
    );
    rdfft=cat(1,rdfft,rdfft(1,:,:)); % periodic repetition
    rdfft=cat(2,rdfft(:,end,:),rdfft); % periodic repetition
    if q==1
        % create new figure
        figure;
        im=imagesc(squeeze(sum(abs(rdfft).^2,3)));
        axis([0.5 2*P+1.5 0.5 8*L+1.5]);
        xticks([1 P+1 2*P+1]);
        xticklabels({num2str(-sensor.maxspeed),'0',num2str(sensor.maxspeed)});
        xlabel('velocity in km/h');
        yticks([1 4*L+1 8*L+1]);
        yticklabels({num2str(sensor.maxrange),num2str(sensor.maxrange/2),'0'});
        ylabel('range in m');
        first=false; % only update graphs in the future
    else
        im.CData=squeeze(sum(abs(rdfft).^2,3));
    end
    drawnow; % update figure
    
    % search for target
    [~,I]=max(squeeze(sum(abs(rdfft(:,P+1,:)).^2,3)));
    fprintf('found target at   %fm\n', (8*L+1-I)*sensor.maxrange/8/L);

    % averaging
    reference=reference+reshape(rdfft(I,P+1,:),12,1,1);
        
end

% stop streaming of radar sensor data
stopstreaming(sensor);

% save calibration data
caldata=1./reference*Q;
save('caldata.mat','caldata');
