function radc=getframe(sensor)
%GETFRAME get next frame from the radar sensor
%   radc        data cube with complex raw adc values
%   sensor      radar sensor device
%
% Lost datagrams are the typical reason for UDP protocol errors.
%
% Due to the reconstruction of the imaginary parts of the raw adc values
% when using the TinyRad only half of the maximum range will be usable.

persistent frame

if isempty(frame)
    frame=0; % reset frame counter
end

persistent dataset

if isempty(dataset)
    dataset=1; % reset dataset counter
end

switch sensor.mode
    case 'adisimo'
        data=sensor.Brd.BrdGetData(); % get data
        lastframe=frame;
        frame=data(1,1); % frame number
        
        % check for missing frames
        if (frame-lastframe)*sensor.sweeprepetitiontime*1e-8~=sensor.framerepetitiontime*1e-3
            warning('missing frames');
        end
        
        % save data to HDF5 file
        info=h5info(sensor.file,'/If');
        h5write(sensor.file,'/If',data(:),[1 info.Dataspace.Size(2)+1],[sensor.samples*sensor.sweeps*4 1]);

        % build the data cube
        radc=reshape(data,sensor.samples,sensor.sweeps,4);
        radc=radc(2:end,:,:); % remove frame numbers
        radc=conj(hilbert(radc)); % reconstruct imaginary parts
        radc=radc.*repmat(reshape(sensor.caldata(1:4),1,1,4),sensor.samples-1,sensor.sweeps,1); % calibration

    case 'adimimo'
        data=sensor.Brd.BrdGetData(); % get data
        lastframe=frame;
        frame=data(1,1); % frame number
        
        % check for missing frames
        if (frame-lastframe)*sensor.sweeprepetitiontime*1e-8~=sensor.framerepetitiontime*1e-3
            warning('missing frames');
        end
        
        % save data to HDF5 file
        info=h5info(sensor.file,'/If');
        h5write(sensor.file,'/If',data(:),[1 info.Dataspace.Size(2)+1],[sensor.samples*sensor.sweeps*8 1]);

        % build the data cube
        radc=reshape(data,sensor.samples,2*sensor.sweeps,4);
        radc=cat(3,radc(:,1:2:end,:),radc(:,2:2:end,:));
        radc=radc(2:end,:,:); % remove frame numbers
        radc=conj(hilbert(radc)); % reconstruct imaginary parts
        radc=radc.*repmat(reshape(sensor.caldata,1,1,8),sensor.samples-1,sensor.sweeps,1); % calibration
        
    case 'adisimofile'
        info=h5info(sensor.file,'/If');
        if dataset<=info.Dataspace.Size(2)
            % load data from HDF5 file
            data=h5read(sensor.file,'/If',[1 dataset],[sensor.samples*sensor.sweeps*4 1]);
            dataset=dataset+1;
            lastframe=frame;
            frame=data(1); % frame number

            % check for missing frames
            if (frame-lastframe)*sensor.sweeprepetitiontime*1e-8~=sensor.framerepetitiontime*1e-3
                warning('missing frames');
            end

            % build the data cube
            radc=reshape(data,sensor.samples,sensor.sweeps,4);
            radc=radc(2:end,:,:); % remove frame numbers
            radc=conj(hilbert(radc)); % reconstruct imaginary parts
            radc=radc.*repmat(reshape(sensor.caldata(1:4),1,1,4),sensor.samples-1,sensor.sweeps,1); % calibration
        else
            radc=[]; % reached end of file
        end

    case 'adimimofile'
        info=h5info(sensor.file,'/If');
        if dataset<=info.Dataspace.Size(2)
            % load data from HDF5 file
            data=h5read(sensor.file,'/If',[1 dataset],[sensor.samples*sensor.sweeps*8 1]);
            dataset=dataset+1;
            lastframe=frame;
            frame=data(1,1); % frame number

            % check for missing frames
            if (frame-lastframe)*sensor.sweeprepetitiontime*1e-8~=sensor.framerepetitiontime*1e-3
                warning('missing frames');
            end

            % build the data cube
            radc=reshape(data,sensor.samples,2*sensor.sweeps,4);
            radc=cat(3,radc(:,1:2:end,:),radc(:,2:2:end,:));
            radc=radc(2:end,:,:); % remove frame numbers
            radc=conj(hilbert(radc)); % reconstruct imaginary parts
            radc=radc.*repmat(reshape(sensor.caldata,1,1,8),sensor.samples-1,sensor.sweeps,1); % calibration
        else
            radc=[]; % reached end of file
        end

    case 'rfbsimo'
        complete=false;
        iqdata=zeros(2,32768); % allocate memory
        while ~complete
            % get RADC message
            data=read(sensor.data,1,'int16');
            len=numel(data(1).Data);
            if len<4||data(1).Data(1)~=typecast(uint8('RA'),'int16') ...
                    ||data(1).Data(2)~=typecast(uint8('DC'),'int16') ...
                    ||data(1).Data(3)~=0||data(1).Data(4)~=2 ...
                    ||len-4>65536
                warning('RADC message expected');
                resynchronize(sensor); % resynchronize
                continue
            end
            iqdata(1:len-4)=data(1).Data(5:len);
            pos=len-4;
            while pos<65536
                data=read(sensor.data,1,'int16'); % get another datagram
                len=numel(data(1).Data);
                if pos+len<=65536
                    iqdata(pos+1:pos+len)=data(1).Data(1:len);
                end
                pos=pos+len;
            end
            if pos>65536
                warning('datagram size not matching');
                resynchronize(sensor); % resynchronize
                continue
            end

            % build the data cube
            % L=128 samples in fast time domain
            % P=64 samples in slow time domain
            % N=4 receive antenna elements
            radc=permute(reshape(iqdata(1,:)+j*iqdata(2,:),128,4,64),[1 3 2]);
            radc=radc.*repmat(reshape(sensor.caldata(1:4),1,1,4),128,64,1); % calibration

            % get DONE message
            data=read(sensor.data,1,'uint32'); % get datagram
            if numel(data(1).Data)~=3 ...
                    ||data(1).Data(1)~=typecast(uint8('DONE'),'uint32') ...
                    ||data(1).Data(2)~=4
                warning('DONE message expected');
                resynchronize(sensor); % resynchronize
                continue
            end        
            lastframe=frame;
            frame=data(1).Data(3); % frame number

            % check for missing frames
            if frame-lastframe~=1
                warning('missing frames');
            end

            % received complete frame without errors
            complete=true; 
            
            % save RADC message
            fwrite(sensor.file,[uint8('RADC'),0,0,2,0]);
            fwrite(sensor.file,iqdata,'int16');

            % save DONE message
            fwrite(sensor.file,[uint8('DONE'),4,0,0,0]);
            fwrite(sensor.file,frame,'uint32');            
            
        end

    case 'rfbmimo'
        complete=false;
        iqdata=zeros(2,49152); % allocate memory
        while ~complete
            % get RADC message
            data=read(sensor.data,1,'int16');
            len=numel(data(1).Data);
            if len<4||data(1).Data(1)~=typecast(uint8('RA'),'int16') ...
                    ||data(1).Data(2)~=typecast(uint8('DC'),'int16') ...
                    ||data(1).Data(3)~=0||data(1).Data(4)~=3 ...
                    ||len-4>98304
                warning('RADC message expected');
                resynchronize(sensor); % resynchronize
                continue
            end
            iqdata(1:len-4)=data(1).Data(5:len);
            pos=len-4;
            while pos<98304
                data=read(sensor.data,1,'int16'); % get another datagram
                len=numel(data(1).Data);
                if pos+len<=98304
                    iqdata(pos+1:pos+len)=data(1).Data(1:len);
                end
                pos=pos+len;
            end
            if pos>98304
                warning('datagram size not matching');
                resynchronize(sensor); % resynchronize
                continue
            end

            % build the data cube
            % L=128 samples in fast time domain
            % P=32 samples in slow time domain
            % N=12 virtual antenna elements
            radc=permute(reshape(iqdata(1,:)+j*iqdata(2,:),128,12,32),[1 3 2]);
            radc=radc.*repmat(reshape(sensor.caldata,1,1,12),128,32,1); % calibration

            % get DONE message
            data=read(sensor.data,1,'uint32'); % get datagram
            if numel(data(1).Data)~=3 ...
                    ||data(1).Data(1)~=typecast(uint8('DONE'),'uint32') ...
                    ||data(1).Data(2)~=4
                warning('DONE message expected');
                resynchronize(sensor); % resynchronize
                continue
            end        
            lastframe=frame;
            frame=data(1).Data(3); % frame number

            % check for missing frames
            if frame-lastframe~=1
                warning('missing frames');
            end

            % received complete frame without errors
            complete=true; 
            
            % save RADC message
            fwrite(sensor.file,[uint8('RADC'),0,0,3,0]);
            fwrite(sensor.file,iqdata,'int16');

            % save DONE message
            fwrite(sensor.file,[uint8('DONE'),4,0,0,0]);
            fwrite(sensor.file,frame,'uint32');            
            
        end

    case 'rfbsimofile'
        iqdata=zeros(2,32768); % allocate memory

        % get RADC message
        [data,count]=fread(sensor.file,8);
        if count~=8
            % reached end of file
            radc=[];
            return
        end
        if ~isequal(data,[uint8('RADC'),0,0,2,0]')
            error('RADC message expected');
        end
        iqdata(:)=fread(sensor.file,65536,'int16');

        % build the data cube
        % L=128 samples in fast time domain
        % P=64 samples in slow time domain
        % N=4 receive antenna elements
        radc=permute(reshape(iqdata(1,:)+j*iqdata(2,:),128,4,64),[1 3 2]);
        radc=radc.*repmat(reshape(sensor.caldata(1:4),1,1,4),128,64,1); % calibration

        % get DONE message
        if ~isequal(fread(sensor.file,8),[uint8('DONE'),4,0,0,0]')
            error('DONE message expected');
        end
        lastframe=frame;
        frame=fread(sensor.file,1,'uint32');
        
        % check for missing frames
        if frame-lastframe~=1
            warning('missing frames');
        end

    case 'rfbmimofile'
        iqdata=zeros(2,49152); % allocate memory

        % get RADC message
        [data,count]=fread(sensor.file,8);
        if count~=8
            % reached end of file
            frame=-1;
            radc=[];
            return
        end
        if ~isequal(data,[uint8('RADC'),0,0,3,0]')
            error('RADC message expected');
        end
        iqdata(:)=fread(sensor.file,98304,'int16');

        % build the data cube
        % L=128 samples in fast time domain
        % P=32 samples in slow time domain
        % N=12 virtual antenna elements
        radc=permute(reshape(iqdata(1,:)+j*iqdata(2,:),128,12,32),[1 3 2]);
        radc=radc.*repmat(reshape(sensor.caldata,1,1,12),128,32,1); % calibration

        % get DONE message
        if ~isequal(fread(sensor.file,8),[uint8('DONE'),4,0,0,0]')
            error('DONE message expected');
        end
        lastframe=frame;
        frame=fread(sensor.file,1,'uint32');
        
        % check for missing frames
        if frame-lastframe~=1
            warning('missing frames');
        end

    otherwise
    	error('unsupported mode');
end

end
