function sensor=startstreaming(mode,file)
%STARTSTREAMING start data streaming from the radar sensor
%   sensor      radar sensor device
%   mode        supported modes
%               'adisimo'       TinyRad, simo mode 
%               'adimimo'       TinyRad, mimo mode
%               'adisimofile'   playback of recorded simo measurements
%               'adimimofile'   playback of recorded mimo measurements
%               'rfbsimo'       V-MD3 in 2D mode 
%               'rfbmimo'       V-MD3 in 3D mode
%               'rfbsimofile'   playback of recorded 2D measurements
%               'rfbmimofile'   playback of recorded 3D measurements
%   file        file name

c0=299792458; % speed of light

clear getframe; % reset counters

switch mode
    case 'adisimo'
        sensor.mode='adisimo';
        sensor.file=file;

        % include necessary directories
        addpath('./DemoRadUsb','./Class');

        % open board object
        sensor.Brd=TinyRad();
        
        % reset previous configuration if board was already configured
        sensor.Brd.BrdRst();
        
        % display software version
        sensor.Brd.BrdDispSwVers();
        
        % configure receiver
        sensor.Brd.RfRxEna();
        
        % configure transmitter
        TxPwr=100;
        sensor.Brd.RfTxEna(1,TxPwr);

        % configure chirp and timing for the measurements
        Cfg.fStrt=24.00e9; % start frequency   
        Cfg.fStop=24.25e9; % stop frequency
        Cfg.TRampUp=128e-6; % chirp duration 
        Cfg.Perd=0.5e-3; % period between measurements
        Cfg.N=128; % number of samples taken at start of chirp 
        Cfg.Seq=[1]; % antenna transmit sequence
        Cfg.CycSiz=2; % number of buffers in the acquisition framework 2
        Cfg.FrmSiz=260; % number of chirp sequences for one measurement cycle
        Cfg.FrmMeasSiz=64; % number of chirp sequences for collecting IF data

        sensor.Brd.RfMeas(Cfg);
                
        % RfGet does not work
        % always returns the values defined in TinyRad.m which are never changed
        % read back of chirp configuration does not work      
       
        % save parameters to HDF5 file
        h5create(file,'/BrdCfg/FreqStrt',1);
        h5write(file,'/BrdCfg/FreqStrt',Cfg.fStrt);
        h5create(file,'/BrdCfg/FreqStop',1);
        h5write(file,'/BrdCfg/FreqStop',Cfg.fStop);
        h5create(file,'/BrdCfg/TimUp',1);
        h5write(file,'/BrdCfg/TimUp',Cfg.TRampUp);
        h5create(file,'/BrdCfg/TimPerd',1);
        h5write(file,'/BrdCfg/TimPerd',Cfg.Perd);
        h5create(file,'/BrdCfg/N',1);
        h5write(file,'/BrdCfg/N',Cfg.N);
        h5create(file,'/BrdCfg/Np',1);
        h5write(file,'/BrdCfg/Np',Cfg.FrmMeasSiz);
        h5create(file,'/BrdCfg/TInt',1);
        h5write(file,'/BrdCfg/TInt',Cfg.FrmSiz*Cfg.Perd);
        h5create(file,'/If',[Cfg.N*Cfg.FrmMeasSiz*4 Inf],'ChunkSize',[Cfg.N*Cfg.FrmMeasSiz*4 1]); % data will go here
        
        % save parameters for later use
        sensor.samples=Cfg.N;      
        sensor.sweeps=Cfg.FrmMeasSiz;
        sensor.sweeprepetitiontime=Cfg.Perd/1e-8; % in 1e-8 s
        sensor.framerepetitiontime=Cfg.FrmSiz*Cfg.Perd/1e-3; % in ms
        sensor.maxrange=c0*1e6/2/(Cfg.fStop-Cfg.fStrt)*Cfg.TRampUp; % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/(Cfg.fStop+Cfg.fStrt)*2/4/Cfg.Perd*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);
        sensor.positions=[0 0 0; 1 0 0; 2 0 0; 3 0 0]*0.5; % normalized positions of the antenna elements
        
        % check data rate
        fprintf('Data rate:     %fMbit/s\n',16*4*Cfg.N*Cfg.FrmMeasSiz./Cfg.Perd/Cfg.FrmSiz/1e6);
        
        % read calibration data
        sensor.caldata=sensor.Brd.BrdGetCalDat();

        % save calibration data to HDF5 file
        h5create(file,'/Cal',[8 2]);
        h5write(file,'/Cal',[real(sensor.caldata) imag(sensor.caldata)]);

    case 'adimimo'
        sensor.mode='adimimo';
        sensor.file=file;

        % include necessary directories
        addpath('./DemoRadUsb','./Class');

        % open board object
        sensor.Brd=TinyRad();
        
        % reset previous configuration if board was already configured
        sensor.Brd.BrdRst();

        % display software version
        sensor.Brd.BrdDispSwVers();
        
        % configure receiver
        sensor.Brd.RfRxEna();
        
        % configure transmitter
        TxPwr=100;
        sensor.Brd.RfTxEna(1,TxPwr);

        % configure chirp and timing for the measurements
        Cfg.fStrt=24.00e9; % start frequency   
        Cfg.fStop=24.25e9; % stop frequency
        Cfg.TRampUp=128e-6; % chirp duration 
        Cfg.Perd=0.5e-3; % period between measurements
        Cfg.N=128; % number of samples taken at start of chirp 
        Cfg.Seq=[1, 2]; % antenna transmit sequence
        Cfg.CycSiz=2; % number of buffers in the acquisition framework 2
        Cfg.FrmSiz=130; % number of chirp sequences for one measurement cycle
        Cfg.FrmMeasSiz=32; % number of chirp sequences for collecting IF data

        sensor.Brd.RfMeas(Cfg);
        
        % RfGet does not work
        % always returns the values defined in TinyRad.m which are never changed
        % read back of chirp configuration does not work      

        % save parameters to HDF5 file
        h5create(file,'/BrdCfg/FreqStrt',1);
        h5write(file,'/BrdCfg/FreqStrt',Cfg.fStrt);
        h5create(file,'/BrdCfg/FreqStop',1);
        h5write(file,'/BrdCfg/FreqStop',Cfg.fStop);
        h5create(file,'/BrdCfg/TimUp',1);
        h5write(file,'/BrdCfg/TimUp',Cfg.TRampUp);
        h5create(file,'/BrdCfg/TimPerd',1);
        h5write(file,'/BrdCfg/TimPerd',Cfg.Perd);
        h5create(file,'/BrdCfg/N',1);
        h5write(file,'/BrdCfg/N',Cfg.N);
        h5create(file,'/BrdCfg/Np',1);
        h5write(file,'/BrdCfg/Np',Cfg.FrmMeasSiz);
        h5create(file,'/BrdCfg/TInt',1);
        h5write(file,'/BrdCfg/TInt',Cfg.FrmSiz*Cfg.Perd*2);
        h5create(file,'/If',[Cfg.N*Cfg.FrmMeasSiz*8 Inf],'ChunkSize',[Cfg.N*Cfg.FrmMeasSiz*8 1]); % data will go here

        % save parameters for later use
        sensor.samples=Cfg.N;      
        sensor.sweeps=Cfg.FrmMeasSiz;
        sensor.sweeprepetitiontime=Cfg.Perd/1e-8; % in 1e-8 s
        sensor.framerepetitiontime=Cfg.FrmSiz*Cfg.Perd*2/1e-3; % in ms
        sensor.maxrange=c0*1e6/2/(Cfg.fStop-Cfg.fStrt)*Cfg.TRampUp; % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/(Cfg.fStop+Cfg.fStrt)*2/4/Cfg.Perd/2*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);
        sensor.positions=[0 0 0; 1 0 0; 2 0 0; 3 0 0; ...
            3 0 0; 4 0 0; 5 0 0; 6 0 0]*0.5; % normalized positions of the virtual antenna elements       
        
        % check data rate
        fprintf('Data rate:     %fMbit/s\n',16*4*Cfg.N*Cfg.FrmMeasSiz./Cfg.Perd/Cfg.FrmSiz/1e6);

        % read calibration data
        sensor.caldata=sensor.Brd.BrdGetCalDat();
        
        % save calibration data to HDF5 file
        h5create(file,'/Cal',[8 2]);
        h5write(file,'/Cal',[real(sensor.caldata) imag(sensor.caldata)]);

    case 'adisimofile'
        sensor.mode='adisimofile';
        sensor.file=file;

        % load parameters from HDF5 file
        sensor.samples=h5read(file,'/BrdCfg/N');      
        sensor.sweeps=h5read(file,'/BrdCfg/Np');
        sensor.sweeprepetitiontime=h5read(file,'/BrdCfg/TimPerd')/1e-8; % in 1e-8 s
        sensor.framerepetitiontime=h5read(file,'/BrdCfg/TInt')/1e-3; % in ms
        sensor.maxrange=c0*1e6/2/(h5read(file,'/BrdCfg/FreqStop')-h5read(file,'/BrdCfg/FreqStrt')) ...
            *h5read(file,'/BrdCfg/TimUp'); % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/(h5read(file,'/BrdCfg/FreqStop')+h5read(file,'/BrdCfg/FreqStrt')) ...
            *2/4/h5read(file,'/BrdCfg/TimPerd')*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);
        sensor.positions=[0 0 0; 1 0 0; 2 0 0; 3 0 0]*0.5; % normalized positions of the antenna elements
        
        % load calibration data from HDF5 file
        sensor.caldata=h5read(file,'/Cal',[1 1],[8 1])+j*h5read(file,'/Cal',[1 2],[8 1]);
        
    case 'adimimofile'
        sensor.mode='adimimofile';
        sensor.file=file;
        
        % load parameters from HDF5 file
        sensor.samples=h5read(file,'/BrdCfg/N');      
        sensor.sweeps=h5read(file,'/BrdCfg/Np');
        sensor.sweeprepetitiontime=h5read(file,'/BrdCfg/TimPerd')/1e-8; % in 1e-8 s
        sensor.framerepetitiontime=h5read(file,'/BrdCfg/TInt')/1e-3; % in ms
        sensor.maxrange=c0*1e6/2/(h5read(file,'/BrdCfg/FreqStop')-h5read(file,'/BrdCfg/FreqStrt')) ...
            *h5read(file,'/BrdCfg/TimUp'); % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/(h5read(file,'/BrdCfg/FreqStop')+h5read(file,'/BrdCfg/FreqStrt')) ...
            *2/4/h5read(file,'/BrdCfg/TimPerd')/2*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);
        sensor.positions=[0 0 0; 1 0 0; 2 0 0; 3 0 0; ...
            3 0 0; 4 0 0; 5 0 0; 6 0 0]*0.5; % normalized positions of the virtual antenna elements       

        % load calibration data from HDF5 file
        sensor.caldata=h5read(file,'/Cal',[1 1],[8 1])+j*h5read(file,'/Cal',[1 2],[8 1]);

    case 'rfbsimo'
        sensor.mode='rfbsimo';

        % open file
        sensor.file=fopen(file,'w');
        
        % open TCP connection
        sensor.cmd=tcpclient('192.168.100.201',6172);
        
        % initialize radar sensor
        write(sensor.cmd,[uint8('INIT'),0,0,0,0]); % start connection
        checkresponse(sensor);
        
        % configure Radar
        write(sensor.cmd,[uint8('RSET'),4,0,0,0,0,0,0,0]); % set mode, 6m, 10km/h
%         write(sensor.cmd,[uint8('RSET'),4,0,0,0,1,0,0,0]); % set mode, 10m, 10km/h
%         write(sensor.cmd,[uint8('RSET'),4,0,0,0,2,0,0,0]); % set mode, 30m, 30km/h
%         write(sensor.cmd,[uint8('RSET'),4,0,0,0,3,0,0,0]); % set mode, 30m, 50km/h
%         write(sensor.cmd,[uint8('RSET'),4,0,0,0,4,0,0,0]); % set mode, 50m, 50km/h
%         write(sensor.cmd,[uint8('RSET'),4,0,0,0,5,0,0,0]); % set mode, 100m, 100km/h
        checkresponse(sensor);

        % show radar software version
        write(sensor.cmd,[uint8('GRPS'),0,0,0,0]); % get radar parameter structure
        checkresponse(sensor);
        if ~isequal(read(sensor.cmd,4),uint8('RPST'))
            error('RPST message expected');
        end
        if read(sensor.cmd,1,'uint32')~=57
            error('57 bytes payload expected');
        end
        rpst=read(sensor.cmd,41); % changeable parameters
        fprintf('Application:   %s\n',char(rpst(1:18)));
        fprintf('FPGA-Version:  %u\n',rpst(20));
        
        % save radar parameter structure
        fwrite(sensor.file,[uint8('RPST'),57,0,0,0]); 
        fwrite(sensor.file,rpst); 
        
        % information parameters
        sensor.startfrequency=double(read(sensor.cmd,1,'uint16')); % in MHz
        sensor.slope=double(read(sensor.cmd,1,'uint16')); % in kHz/us
        sensor.samples=double(read(sensor.cmd,1,'uint16'));
        sensor.sweeps=double(read(sensor.cmd,1,'uint16'));
        sensor.samplerate=double(read(sensor.cmd,1,'uint16')); % in ksps
        sensor.sweeprepetitiontime=double(read(sensor.cmd,1,'uint16')); % in 1e-8 s
        sensor.framerepetitiontime=double(read(sensor.cmd,1,'uint16')); % in ms
        sensor.transmitantennaindex=double(read(sensor.cmd,1));
        sensor.calibrationstatus=double(read(sensor.cmd,1));
        
        % save information parameters
        fwrite(sensor.file,sensor.startfrequency,'uint16'); % in MHz
        fwrite(sensor.file,sensor.slope,'uint16'); % in kHz/us
        fwrite(sensor.file,sensor.samples,'uint16');
        fwrite(sensor.file,sensor.sweeps,'uint16');
        fwrite(sensor.file,sensor.samplerate,'uint16'); % in ksps
        fwrite(sensor.file,sensor.sweeprepetitiontime,'uint16'); % in 1e-8 s
        fwrite(sensor.file,sensor.framerepetitiontime,'uint16'); % in ms
        fwrite(sensor.file,sensor.transmitantennaindex);
        fwrite(sensor.file,sensor.calibrationstatus);        

        % measurement range
        sensor.maxrange=c0/2/sensor.slope/1e9*sensor.samplerate*1e3; % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/4/sensor.startfrequency/1e6/sensor.sweeprepetitiontime/1e-8*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);
        
        % normalized positions of the antenna elements
        sensor.positions=[3 0 0; 2 0 0; 1 0 0; 0 0 0] ...
            *0.0024638*(sensor.startfrequency*1e6+ ...
            sensor.slope*1e9*sensor.samples/sensor.samplerate/1e3/2)/c0; 

        % load calibration data
        load('caldata.mat');
        sensor.caldata=caldata;

        % open UDP connection
        sensor.data=udpport('datagram','LocalPort',4567);
 
        % start streaming
        write(sensor.cmd,[uint8('RDOT'),4,0,0,0,0x21,0,0,0]); % output RADC / DONE
        checkresponse(sensor);

    case 'rfbmimo'
        sensor.mode='rfbmimo';
        
        % open file
        sensor.file=fopen(file,'w');
        
        % open TCP connection
        sensor.cmd=tcpclient('192.168.100.201',6172);
        
        % initialize radar sensor
        write(sensor.cmd,[uint8('INIT'),0,0,0,0]); % start connection
        checkresponse(sensor);
        
        % configure Radar
        write(sensor.cmd,[uint8('RSET'),4,0,0,0,6,0,0,0]); % set mode, 6m, 10km/h
%         write(sensor.cmd,[uint8('RSET'),4,0,0,0,7,0,0,0]); % set mode, 10m, 10km/h
%         write(sensor.cmd,[uint8('RSET'),4,0,0,0,8,0,0,0]); % set mode, 30m, 30km/h
        checkresponse(sensor);

        % show radar software version
        write(sensor.cmd,[uint8('GRPS'),0,0,0,0]); % get radar parameter structure
        checkresponse(sensor);
        if ~isequal(read(sensor.cmd,4),uint8('RPST'))
            error('RPST message expected');
        end
        if read(sensor.cmd,1,'uint32')~=57
            error('57 bytes payload expected');
        end
        rpst=read(sensor.cmd,41); % changeable parameters
        fprintf('Application:   %s\n',char(rpst(1:18)));
        fprintf('FPGA-Version:  %u\n',rpst(20));
        
        % save radar parameter structure
        fwrite(sensor.file,[uint8('RPST'),57,0,0,0]); 
        fwrite(sensor.file,rpst); 
        
        % information parameters
        sensor.startfrequency=double(read(sensor.cmd,1,'uint16')); % in MHz
        sensor.slope=double(read(sensor.cmd,1,'uint16')); % in kHz/us
        sensor.samples=double(read(sensor.cmd,1,'uint16'));
        sensor.sweeps=double(read(sensor.cmd,1,'uint16'));
        sensor.samplerate=double(read(sensor.cmd,1,'uint16')); % in ksps
        sensor.sweeprepetitiontime=double(read(sensor.cmd,1,'uint16')); % in 1e-8 s
        sensor.framerepetitiontime=double(read(sensor.cmd,1,'uint16')); % in ms
        sensor.transmitantennaindex=double(read(sensor.cmd,1));
        sensor.calibrationstatus=double(read(sensor.cmd,1));

        % save information parameters
        fwrite(sensor.file,sensor.startfrequency,'uint16'); % in MHz
        fwrite(sensor.file,sensor.slope,'uint16'); % in kHz/us
        fwrite(sensor.file,sensor.samples,'uint16');
        fwrite(sensor.file,sensor.sweeps,'uint16');
        fwrite(sensor.file,sensor.samplerate,'uint16'); % in ksps
        fwrite(sensor.file,sensor.sweeprepetitiontime,'uint16'); % in 1e-8 s
        fwrite(sensor.file,sensor.framerepetitiontime,'uint16'); % in ms
        fwrite(sensor.file,sensor.transmitantennaindex);
        fwrite(sensor.file,sensor.calibrationstatus);

        % measurement range
        sensor.maxrange=c0/2/sensor.slope/1e9*sensor.samplerate*1e3; % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/4/sensor.startfrequency/1e6/sensor.sweeprepetitiontime/1e-8/3*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);
        
        % normalized positions of the virtual antenna elements
        sensor.positions=[7 0 0; 6 0 0; 5 0 0; 4 0 0; ...
            5 0 1; 4 0 1; 3 0 1; 2 0 1; ...
            3 0 0; 2 0 0; 1 0 0; 0 0 0] ...
            *0.0024638*(sensor.startfrequency*1e6+ ...
            sensor.slope*1e9*sensor.samples/sensor.samplerate/1e3/2)/c0; 

        % load calibration data
        load('caldata.mat');
        sensor.caldata=caldata;

        % open UDP connection
        sensor.data=udpport('datagram','LocalPort',4567);
 
        % start streaming
        write(sensor.cmd,[uint8('RDOT'),4,0,0,0,0x21,0,0,0]); % output RADC / DONE
        checkresponse(sensor);

    case 'rfbsimofile'
        sensor.mode='rfbsimofile';
        
        % open file
        sensor.file=fopen(file);
                
        disp('read data from file');
        
        % get radar parameter structure
        if ~isequal(fread(sensor.file,4),uint8('RPST')')
            error('RPST message expected');
        end
        if fread(sensor.file,1,'uint32')~=57
            error('57 bytes payload expected');
        end
        rpst=fread(sensor.file,41); % changeable parameters
        fprintf('Application:   %s\n',char(rpst(1:18)));
        fprintf('FPGA-Version:  %u\n',rpst(20));
        
        % information parameters
        sensor.startfrequency=fread(sensor.file,1,'uint16'); % in MHz
        sensor.slope=fread(sensor.file,1,'uint16'); % in kHz/us
        sensor.samples=fread(sensor.file,1,'uint16');
        sensor.sweeps=fread(sensor.file,1,'uint16');
        sensor.samplerate=fread(sensor.file,1,'uint16'); % in ksps
        sensor.sweeprepetitiontime=fread(sensor.file,1,'uint16'); % in 1e-8 s
        sensor.framerepetitiontime=fread(sensor.file,1,'uint16'); % in ms
        sensor.transmitantennaindex=fread(sensor.file,1);
        sensor.calibrationstatus=fread(sensor.file,1);

        % measurement range
        sensor.maxrange=c0/2/sensor.slope/1e9*sensor.samplerate*1e3; % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/4/sensor.startfrequency/1e6/sensor.sweeprepetitiontime/1e-8*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);

        % normalized positions of the antenna elements
        sensor.positions=[3 0 0; 2 0 0; 1 0 0; 0 0 0] ...
            *0.0024638*(sensor.startfrequency*1e6+ ...
            sensor.slope*1e9*sensor.samples/sensor.samplerate/1e3/2)/c0; 

        % load calibration data
        load('caldata.mat');
        sensor.caldata=caldata;

    case 'rfbmimofile'
        sensor.mode='rfbmimofile';
        
        % open file
        sensor.file=fopen(file);
        
        disp('read data from file');
        
        % get radar parameter structure
        if ~isequal(fread(sensor.file,4),uint8('RPST')')
            error('RPST message expected');
        end
        if fread(sensor.file,1,'uint32')~=57
            error('57 bytes payload expected');
        end
        rpst=fread(sensor.file,41); % changeable parameters
        fprintf('Application:   %s\n',char(rpst(1:18)));
        fprintf('FPGA-Version:  %u\n',rpst(20));
        
        % information parameters
        sensor.startfrequency=fread(sensor.file,1,'uint16'); % in MHz
        sensor.slope=fread(sensor.file,1,'uint16'); % in kHz/us
        sensor.samples=fread(sensor.file,1,'uint16');
        sensor.sweeps=fread(sensor.file,1,'uint16');
        sensor.samplerate=fread(sensor.file,1,'uint16'); % in ksps
        sensor.sweeprepetitiontime=fread(sensor.file,1,'uint16'); % in 1e-8 s
        sensor.framerepetitiontime=fread(sensor.file,1,'uint16'); % in ms
        sensor.transmitantennaindex=fread(sensor.file,1);
        sensor.calibrationstatus=fread(sensor.file,1);

        % measurement range
        sensor.maxrange=c0/2/sensor.slope/1e9*sensor.samplerate*1e3; % in m
        fprintf('Maximum range: %fm\n',sensor.maxrange);
        sensor.maxspeed=c0/4/sensor.startfrequency/1e6/sensor.sweeprepetitiontime/1e-8/3*3.6; % in km/h
        fprintf('Maximum speed: %fkm/h\n',sensor.maxspeed);

        % normalized positions of the virtual antenna elements
        sensor.positions=[7 0 0; 6 0 0; 5 0 0; 4 0 0; ...
            5 0 1; 4 0 1; 3 0 1; 2 0 1; ...
            3 0 0; 2 0 0; 1 0 0; 0 0 0] ...
            *0.0024638*(sensor.startfrequency*1e6+ ...
            sensor.slope*1e9*sensor.samples/sensor.samplerate/1e3/2)/c0; 

        % load calibration data
        load('caldata.mat');
        sensor.caldata=caldata;

    otherwise
    	error('unsupported mode');
end

end
